/*
    youtubeDownloadPlugin

		Copyright (c) 2008-2009 by Alexander Kazarin <boiler@co.ru>

 ***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************
*/

#include "ytdownload.h"

#include <QAction>
#include <QSettings>

#include <QNetworkAccessManager>
#include <QNetworkRequest>
#include <QNetworkReply>

#include <QDebug>

bool urlpreviewPlugin::init(PluginSystemInterface *plugin_system)
{
	qRegisterMetaType<TreeModelItem>("TreeModelItem");

	PluginInterface::init(plugin_system);
        m_plugin_icon = new QIcon(":/icons/youtube.png");
	m_plugin_system = plugin_system;

	netman = new QNetworkAccessManager();
        connect(netman, SIGNAL(finished(QNetworkReply*)), SLOT(netmanFinished(QNetworkReply*)));

	m_event_msgin = m_plugin_system->registerEventHandler("Core/ChatWindow/ReceiveLevel3", this);
	m_event_msgout = m_plugin_system->registerEventHandler("Core/ChatWindow/SendLevel1", this);
        qDebug() << "YouTube: init";
	return true;
}

QIcon *urlpreviewPlugin::icon()
{
	return m_plugin_icon;
}

void urlpreviewPlugin::saveSettings()
{
	settingswidget->saveSettings();
}

void urlpreviewPlugin::loadSettings()
{
        QSettings settings(QSettings::IniFormat, QSettings::UserScope, "qutim/qutim."+m_profile_name, "plugin_youtube");

        QString template_default = "(<a href=\"%URL%\">"+tr("Download")+"</a>)";
	if (settings.value("template").isNull()) settings.setValue("template", template_default);

	cfg_enable_in = settings.value("enable_in", true).toBool();
	cfg_enable_out = settings.value("enable_out", false).toBool();
        cfg_str["template"] = settings.value("template", template_default).toString();
}

QWidget *urlpreviewPlugin::settingsWidget()
{
	settingswidget = new urlpreviewSettings(m_profile_name);
	connect(settingswidget, SIGNAL(settingsChanged()), this, SLOT(loadSettings()));
	return settingswidget;
}

void urlpreviewPlugin::removeSettingsWidget()
{
	delete settingsWidget();
}

void urlpreviewPlugin::setProfileName(const QString &profile_name)
{
	m_profile_name = profile_name;
	loadSettings();
}


void urlpreviewPlugin::processEvent(Event &event)
{
        //Check, if event type is incoming or outgoing messages and if they're enabled in config
	if ((event.id==m_event_msgin && cfg_enable_in) ||
			(event.id==m_event_msgout && cfg_enable_out)) {

		TreeModelItem item = *(TreeModelItem*)(event.args.at(0));
		QString *msg = (QString*)(event.args.at(1));

                //Ugly regexp is horrible but working
                QRegExp urlRegExp("(\\<a\\s+href='(([^'\\s<]+youtube.com/v/)[^\\s&'<]+|([^'\\s<]+youtube.com/watch\\?v=)[^\\s&'<]+)[^<]+</a>)");
		int pos = 0;

                //Offset is used because of multiple itserts - position, returned by regexp.indexin doesn't handle text shifts during them
                int offsetLength = 0;
		QString smsg = *msg;
		QString rmsg = *msg;

                //Loop through text to find all matching urls
                while ((pos = urlRegExp.indexIn(smsg, pos)) != -1) {
                        QString url = urlRegExp.cap(2);

			QString uid;
                        //Generate unique key for each found url, matching pattern
			while (uid.isEmpty() || uidurls.contains(uid)) {
				uid = QString::number(rand() % 0x8000);
			}

                       if(!url.isEmpty()) {
                           url.replace(urlRegExp.cap(3), "");
                           url.replace(urlRegExp.cap(4), "");
                        } //url will contain video_id now.

			uidurls[uid] = url;
			uiditems[uid] = item;

                        pos += urlRegExp.matchedLength();

                        //This element will contain value after script processing is done
                        QString addition = "&nbsp;<span id='ytdownload"+uid+"'></span>";
                        rmsg.insert(pos + offsetLength, addition);
                        offsetLength += addition.length();

			QNetworkRequest netreq;
                        //Get text, containing token for each found url
                        netreq.setUrl(QUrl("http://www.youtube.com/get_video_info.php?video_id="+url));
                        netreq.setRawHeader("Ranges", "bytes=0-0");
                        //Do request
                        netman->get(netreq);

                        //qDebug()<<"YouTube: request"<<url<<uid<<rmsg << pos << "\r\n\r\n";
		}
		*msg = rmsg;
	}
}

void urlpreviewPlugin::netmanFinished( QNetworkReply* reply )
{
    //Sometimes youtube makes redirect to other location.
    QUrl redirect = reply->attribute(QNetworkRequest::RedirectionTargetAttribute).toUrl();
    if(!redirect.isEmpty()) {
        QNetworkRequest netreq(redirect);
        netman->get(netreq);
    } else {

        //Magic... didn't touch it
	QString type = reply->rawHeader("Content-Type");
	quint64 size=0;
	if (reply->rawHeaderList().contains("Content-Range")) {
		QString rangestr = reply->rawHeader("Content-Range");
		QRegExp rangerx("/(\\d+)");
		if (rangerx.indexIn(rangestr)>-1) {
			size = rangerx.cap(1).toInt();
		}
	}
	else if (reply->rawHeaderList().contains("Content-Length")) {
		size = reply->rawHeader("Content-Length").toInt();
	}

	if (!type.isEmpty() && size>0) {
            //Read all contents of reply
            QString info = reply->readAll();

            foreach (QString uid, uidurls.keys()) {
                QString url = reply->url().toString();
                //qDebug() << "YouTube:  processing" << url << uid << uidurls[uid].isEmpty() << url.contains(uidurls[uid]);

                //As the key is video_id then we need to check if processed url contains it. In case multiple occurence of same url no matter in what order
                //they're processed - one url is removed during every loop
                if (!uidurls[uid].isEmpty() && url.contains(uidurls[uid])) {
                    QRegExp tokenRegExp("&token=[^\\s&]+");
                    QRegExp idRegExp("video_id=[^\\s&]+");

                    QString result;

                    if(tokenRegExp.indexIn(info, 0) != -1) {
                        //qDebug() << tokenRegExp.cap(0);
                        if(idRegExp.indexIn(info, 0) != -1) {
                            //Result will be formed only if both video_id and token are captured by regexes
                            result = "http://www.youtube.com/get_video.php?"+ idRegExp.cap(0) +tokenRegExp.cap(0).replace("token", "t");
                        }
                    }

                    //Load template and add url to it
                    QString pstr = cfg_str["template"];
                    pstr.replace("%URL%", result);

                    if(!result.isEmpty()) {
                        QString js = "ytdownload"+uid+".innerHTML = \""+pstr.replace("\"", "\\\"")+"\";";
                        TreeModelItem item = uiditems[uid];

                        //Use javascript to add text after it's displayed. Used to avoid delays.
                        quint16 event_evaljs_id = SystemsCity::PluginSystem()->registerEventHandler("Core/ChatWindow/EvaluateJavaScript");

                        //Some magic from Boiler... just don't touch
                        if (item.m_item_type==34) {
                            item.m_item_type = 32;
                            item.m_item_name = item.m_parent_name;
                            item.m_parent_name = item.m_account_name;
                        }

                        Event(event_evaljs_id, 2, &item, &js).send(); //Send event notification
                    }

                    //Do cleanup and remove url from list to avoid processing multiple times
                    uidurls[uid].clear();
                    uidurls[uid].~QString();
                    uidurls.remove(uid);
                    uiditems.remove(uid);
                }

            }

        }
    }

    reply->close();
    delete reply;

}


Q_EXPORT_PLUGIN2(urlpreviewPlugin,urlpreviewPlugin);
